from typing import Optional as _Optional

from FinanceDataReader import DataReader as _DataReader
from pandas import DataFrame as _DataFrame
from pandas import concat as _concat


def timeseries(
    ticker: str, sdate: _Optional[str] = None, edate: _Optional[str] = None
) -> _DataFrame:
    return _DataReader(ticker, sdate, edate, None, None)


def fred_timeseries(
    ticker: str, sdate: _Optional[str] = None, edate: _Optional[str] = None
) -> _DataFrame:
    """FRED: Federal Reserve Economic Data"""
    return _DataReader(ticker, sdate, edate, None, "FRED")


def krx_timeseries(
    ticker: str, sdate: _Optional[str] = None, edate: _Optional[str] = None
) -> _DataFrame:
    """KRX: Korea Exchange"""
    return _DataReader(ticker, sdate, edate, "KRX", None)


def sse_timeseries(
    ticker: str, sdate: _Optional[str] = None, edate: _Optional[str] = None
) -> _DataFrame:
    """SSE: Shanghai Stock Exchange"""
    return _DataReader(ticker, sdate, edate, "SSE", None)


def szse_timeseries(
    ticker: str, sdate: _Optional[str] = None, edate: _Optional[str] = None
) -> _DataFrame:
    """SZSE: Shenzhen Stock Exchange"""
    return _DataReader(ticker, sdate, edate, "SZSE", None)


def hkex_timeseries(
    ticker: str, sdate: _Optional[str] = None, edate: _Optional[str] = None
) -> _DataFrame:
    """HKEX: Hong Kong Exchanges"""
    return _DataReader(ticker, sdate, edate, "HKEX", None)


def tse_timeseries(
    ticker: str, sdate: _Optional[str] = None, edate: _Optional[str] = None
) -> _DataFrame:
    """TSE: Tokyo Stock Exchange"""
    return _DataReader(ticker, sdate, edate, "TSE", None)


def hose_timeseries(
    ticker: str, sdate: _Optional[str] = None, edate: _Optional[str] = None
) -> _DataFrame:
    """HOSE: Ho Chi Minh Stock Exchange"""
    return _DataReader(ticker, sdate, edate, "HOSE", None)


def timeseries_tickers(
    tickers: list[str],
    sdate: _Optional[str] = None,
    edate: _Optional[str] = None,
    exchange: _Optional[str] = None,
) -> _DataFrame:
    df = _DataFrame()
    for ticker in dict.fromkeys(tickers):
        dfi = _DataReader(ticker, sdate, edate, exchange, None)
        if isinstance(dfi, _DataFrame):
            dfi = dfi[["Close"]]
            dfi.columns = [ticker]
            df = _concat([df, dfi], axis=1)
    return df
