from datetime import date as _date
from datetime import datetime as _datetime
from datetime import time as _time
from datetime import timedelta as _timedelta
from time import time as __time
from typing import Optional as _Optional

from pytz import BaseTzInfo as _BaseTzInfo
from pytz import utc as _utc


def now(tz: _Optional[_BaseTzInfo] = None) -> _datetime:
    """
    Return the current timezone datetime.
    """
    utcnow = _utc.localize(_datetime.utcnow())
    if tz is not None:
        return utcnow.astimezone(tz)
    return utcnow


def now_time(tz: _Optional[_BaseTzInfo] = None) -> _time:
    """
    Return the current timezone time.
    """
    return now(tz).timetz()


def now_date(tz: _Optional[_BaseTzInfo] = None) -> _date:
    """
    Return the current timezone date.
    """
    return now(tz).date()


def unix_time() -> float:
    return __time()


def date_add(d: _date, days: int) -> _date:
    """
    days range: -999999999 <= days <= 999999999
    """
    return d + _timedelta(days=days)


def date_weekday(d: _date) -> int:
    """
    Return the day of the week as an integer, Monday is 0 and Sunday is 6.
    """
    return d.weekday()


def date_is_weekend(d: _date) -> bool:
    return date_weekday(d) > 4


def date_to_str(d: _date) -> str:
    """
    Return a string representing the date in ISO 8601 format, YYYY-MM-DD.
    """
    return d.isoformat()


def date_from_str(datestr: str, fmt: str = "%Y-%m-%d") -> _date:
    return _datetime.strptime(datestr, fmt).date()


def today_adds(
    list_days: list[int], tz: _Optional[_BaseTzInfo] = None
) -> list[_date]:
    t = now_date(tz)
    return [date_add(t, days) for days in list_days]


def today_adds_str(
    list_days: list[int], tz: _Optional[_BaseTzInfo] = None
) -> list[str]:
    t = now_date(tz)
    return [date_to_str(date_add(t, days)) for days in list_days]
